package sample.jpa;

import javax.annotation.PostConstruct;
import javax.ejb.Local;
import javax.ejb.Remove;
import javax.ejb.Stateful;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.PersistenceUnit;


@Stateful
@TransactionManagement(TransactionManagementType.CONTAINER)
@Local(AccountInterface.class)
public class AccountBean implements AccountInterface{
	
	@PersistenceUnit(unitName="AccountUnit")
	 private EntityManagerFactory emf;
	
	private EntityManager em;
	
	Account account;
	Person person;
	
	@PostConstruct
	public void init(){
		em = emf.createEntityManager();
	}
	
	public void initialize(int accountNumber){
		
		account = em.find(Account.class, accountNumber);
		if(account == null)throw new IllegalStateException("Account number ("+accountNumber+") not found");

		person = em.find(Person.class, account.getPersonId());
		if(person == null)throw new IllegalStateException("Person Id("+account.getPersonId()+") not found. There is a descripancy in the database for account number ("+accountNumber+")");
	}
	
	public String getPersonName(){
		return person.getPersonName();
	}
	
	public void setPersonName(String personName){
		person.setPersonName(personName);
	}
	
	public String getAddress(){
		return person.getAddress();
	}
	
	public void setAddress(String address){
		person.setAddress(address);
	}
	
	public void updateAllValues(String address, String personName, double balance){
		em.joinTransaction();
		setAddress(address);
		setPersonName(personName);
		setBalance(balance);
	}
	
	public double getBalance(){
		return account.getBalance();
	}

	public void setBalance(double balance){
		account.setBalance(balance);
	}
	
	public void withdraw(double amount){
		if(amount > getBalance())throw new IllegalStateException("The amount("+amount+") to be withdrawn is more than the available balance("+getBalance()+")");
		em.joinTransaction();
		setBalance(getBalance() - amount);
		
	}
	
	public void deposit(double amount){
			em.joinTransaction();
			setBalance(getBalance() + amount);
	}
	
	@Remove
	public void destroy(){
		em.close();
	}
}
